#ifndef __KRTCDEVICEMANAGER_H__
#define __KRTCDEVICEMANAGER_H__

#include "KRtcTypeDef.h"

namespace KRTCSDK {

/// @defgroup KRtcDeviceManager KRtcDeviceManager设备管理类
/// KRTC设备管理接口类
/// @{
class KRtcDeviceManager
{
public:
    virtual ~KRtcDeviceManager(){}

    /**
     * 获取设备列表
     *
     * @param type 设备类型
     *
     * @return  DeviceList对象指针，没有设备时返回nullptr
     * @note 返回的 DeviceList* 需要调用releaseDeviceList接口销毁
     */
    virtual DeviceList* getDeviceList(DEVICETYPE type) = 0;

    /**
     * 释放DeviceList*对象内存
     *
     * @param list DeviceList对象指针
     */
    virtual void releaseDeviceList(DeviceList* list) = 0;

    /**
     * 设置当前设备
     *
     * @param type 设备类型
     * @param deviceId 设备id
     *
     * @return 0：成功 -1：参数错误 -2：设备开启失败
     */
    virtual int32_t setCurrentDevice(DEVICETYPE type, const char* deviceId) = 0;

    /**
     * 获取当前设备
     *
     * @param type 设备类型
     * @return  DeviceInfo对象指针，没有设备时返回nullptr
     * @note 返回的 DeviceInfo* 需要调用releaseDeviceInfo接口销毁
     */
    virtual DeviceInfo* getCurrentDevice(DEVICETYPE type) = 0;

    /**
     * 释放DeviceInfo内存
     *
     * @param info DeviceInfo对象指针
     */
    virtual void releaseDeviceInfo(DeviceInfo* info) = 0;

#if __ANDROID__
    /**
     * 切换前后摄像头
     *
     * @param frontCamera true：前置 false：后置
     * @return  0：成功 -2：摄像头开启失败
     * @note 该接口只支持 Android 平台
     */
    virtual int32_t switchCamera(bool frontCamera) = 0;

    /**
     * 判断当前置摄像头是否为前置摄像头
     *
     * @return  true：前置 false：后置
     * @note 该接口只支持 Android 平台
     */
    virtual bool isFrontCamera() = 0;
#endif

//#if (_WIN32 || defined(_LINUX_))
    /**
     * 设置扬声器音量（0-255）
     *
     * @param volume 音量大小
     * @return  0：设置成功 -1：设置失败
     */
    virtual int32_t setSpeakerVolume(uint32_t volume) = 0;

    /**
     * 获取扬声器音量
     *
     * @return  0-255：当前音量 -1：获取设备音量失败
     */
    virtual int32_t getSpeakerVolume() = 0;

    /**
     * 获取屏幕列表
     *
     * @return  DeviceInfo对象指针，获取失败时返回nullptr
     */
    virtual ScreenList* getScreenList() = 0;

    /**
     * 释放ScreenList内存
     *
     * @param list ScreenList对象指针
     */
    virtual void releaseScreenList(ScreenList* list) = 0;
//#endif

    /**
     * 开始摄像头测试
     *
     * @param deviceId 摄像头设备id，传入nullptr表示使用setCurrentDevice指定的摄像头
     * @param displayWindowId 回显窗口id
     * @param mirror 镜像模式 true：镜像 false：不镜像
     *
     * @return  0：成功 -1：参数错误 -2：开始摄像头设备失败
     */
    virtual int32_t startCameraDeviceTest(const char* deviceId, winId displayWindowId, bool mirror) = 0;

    /**
     * 开始摄像头测试
     *
     * @param deviceId 摄像头设备id，传入nullptr表示使用setCurrentDevice指定的摄像头
     * @param callback 回调函数指针，摄像头采集数据将通过callback回调
     *
     * @return  0：成功 -1：参数错误 -2：开始摄像头设备失败
     */
    virtual int32_t startCameraDeviceTest(const char* deviceId, onLocalVideoData callback) = 0;

    /**
     * 停止摄像头测试
     *
     */
    virtual void stopCameraDeviceTest() = 0;

    /**
     * 设置摄像头装配方向
     *
     * @param deviceId 摄像头设备id，传入nullptr表示使用setCurrentDevice指定的摄像头
     * @param rotation 旋转角度
     *
     * @return  0：成功 -1：参数错误
     */
    virtual int32_t setCameraOrientation(const char* deviceId, CAMERA_CAPTURE_ORIENTATION orientation) = 0;

    /**
     * 开始麦克风测试
     *
     * @param deviceId 麦克风设备id，传入nullptr表示使用setCurrentDevice指定的麦克风
     * @param callback 回调函数，SDK每200ms回调一次，汇报当前音频能量（0-127）
     *
     * @return  0：成功 -1：参数错误 -2：开始麦克风设备失败
     */
    virtual int32_t startMicDeviceTest(const char* deviceId, onLocalAudioLevel callback) = 0;

    /**
     * 开始麦克风测试
     *
     * @param deviceId 麦克风设备id，传入nullptr表示使用setCurrentDevice指定的麦克风
     * @param callback 回调函数，麦克风采集数据将通过callback回调
     *
     * @return  0：成功 -1：参数错误 -2：开始麦克风设备失败
     */
    virtual int32_t startMicDeviceTest(const char* deviceId, onLocalAudioData callback) = 0;

    /**
     * 结束麦克风测试
     *
     */
    virtual void stopMicDeviceTest() = 0;

    /**
     * 设置麦克风音量（1-100）
     *
     * @param volume 音量大小
     * @return  0：设置成功 -1：设置失败
     */
    virtual int32_t setMicrophoneVolume(uint32_t volume) = 0;

    /**
     * 获取麦克风音量
     *
     * @return  1-100：当前音量 -1：获取设备音量失败
     */
    virtual int32_t getMicrophoneVolume() = 0;

    /**
     * 打开本地摄像头
     *
     */
    virtual int32_t openLocalVideo() = 0;

    /**
     * 关闭本地摄像头
     *
     */
    virtual void closeLocalVideo() = 0;
};
/// @}
}

#endif
